@@ -44,8 +44,6 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 	STORE_FIELD(hv, conf, accounting_storage_port, uint16_t);
 	if (conf->accounting_storage_type)
 		STORE_FIELD(hv, conf, accounting_storage_type, charp);
-	if (conf->accounting_storage_user)
-		STORE_FIELD(hv, conf, accounting_storage_user, charp);
 
 	if (conf->authinfo)
 		STORE_FIELD(hv, conf, authinfo, charp);
@@ -77,9 +75,6 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 		av_store(av, i, newSVpv(conf->control_machine[i], 0));
 	hv_store_sv(hv, "control_machine", newRV_noinc((SV*)av));
 
-	if (conf->core_spec_plugin)
-		STORE_FIELD(hv, conf, core_spec_plugin, charp);
-
 	STORE_FIELD(hv, conf, cpu_freq_def, uint32_t);
 
 	if (conf->cred_type)
@@ -93,25 +88,13 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 
 	STORE_FIELD(hv, conf, enforce_part_limits, uint16_t);
 
-	if (conf->epilog)
-		STORE_FIELD(hv, conf, epilog, charp);
 	STORE_FIELD(hv, conf, epilog_msg_time, uint32_t);
-	if (conf->epilog_slurmctld)
-		STORE_FIELD(hv, conf, epilog_slurmctld, charp);
-
-	if (conf->ext_sensors_conf)
-		STORE_FIELD(hv, conf, ext_sensors_conf, charp);
-	STORE_FIELD(hv, conf, ext_sensors_freq, uint16_t);
-	if (conf->ext_sensors_type)
-		STORE_FIELD(hv, conf, ext_sensors_type, charp);
 
 	STORE_FIELD(hv, conf, conf_flags, uint32_t);
 
 	STORE_FIELD(hv, conf, first_job_id, uint32_t);
 	STORE_FIELD(hv, conf, fs_dampening_factor, uint16_t);
 
-	STORE_FIELD(hv, conf, get_env_timeout, uint16_t);
-
 	if (conf->gres_plugins)
 		STORE_FIELD(hv, conf, gres_plugins, charp);
 
@@ -148,8 +131,8 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 	if (conf->job_comp_user)
 		STORE_FIELD(hv, conf, job_comp_user, charp);
 
-	if (conf->job_container_plugin)
-		STORE_FIELD(hv, conf, job_container_plugin, charp);
+	if (conf->namespace_plugin)
+		STORE_FIELD(hv, conf, namespace_plugin, charp);
 
 	STORE_FIELD(hv, conf, job_file_append, uint16_t);
 
@@ -197,9 +180,6 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 
 	STORE_FIELD(hv, conf, next_job_id, uint32_t);
 
-	if (conf->node_prefix)
-		STORE_FIELD(hv, conf, node_prefix, charp);
-
 	STORE_FIELD(hv, conf, over_time_limit, uint16_t);
 
 	if (conf->plugindir)
@@ -208,9 +188,6 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 	if (conf->plugstack)
 		STORE_FIELD(hv, conf, plugstack, charp);
 
-	if (conf->power_parameters)
-		STORE_FIELD(hv, conf, power_parameters, charp);
-
 	STORE_FIELD(hv, conf, preempt_mode, uint16_t);
 
 	if (conf->preempt_params)
@@ -240,11 +217,7 @@ int slurm_ctl_conf_to_hv(slurm_conf_t *conf, HV *hv)
 	if (conf->proctrack_type)
 		STORE_FIELD(hv, conf, proctrack_type, charp);
 
-	if (conf->prolog)
-		STORE_FIELD(hv, conf, prolog, charp);
 	STORE_FIELD(hv, conf, prolog_flags, uint16_t);
-	if (conf->prolog_slurmctld)
-		STORE_FIELD(hv, conf, prolog_slurmctld, charp);
 
 	STORE_FIELD(hv, conf, propagate_prio_process, uint16_t);
 	if (conf->propagate_rlimits)
@@ -399,7 +372,6 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, accounting_storage_pass, charp, FALSE);
 	FETCH_FIELD(hv, conf, accounting_storage_port, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, accounting_storage_type, charp, FALSE);
-	FETCH_FIELD(hv, conf, accounting_storage_user, charp, FALSE);
 
 	FETCH_FIELD(hv, conf, authinfo, charp, FALSE);
 	FETCH_FIELD(hv, conf, authtype, charp, FALSE);
@@ -441,24 +413,17 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, control_addr[1], charp, FALSE);
 	FETCH_FIELD(hv, conf, control_machine[1], charp, FALSE);
 
-	FETCH_FIELD(hv, conf, core_spec_plugin, charp, FALSE);
 	FETCH_FIELD(hv, conf, cpu_freq_def, uint32_t, FALSE);
 	FETCH_FIELD(hv, conf, cred_type, charp, FALSE);
 	FETCH_FIELD(hv, conf, debug_flags, uint64_t, TRUE);
 	FETCH_FIELD(hv, conf, def_mem_per_cpu, uint64_t, TRUE);
 	FETCH_FIELD(hv, conf, eio_timeout, uint16_t, FALSE);
 	FETCH_FIELD(hv, conf, enforce_part_limits, uint16_t, TRUE);
-	FETCH_FIELD(hv, conf, epilog, charp, FALSE);
 	FETCH_FIELD(hv, conf, epilog_msg_time, uint32_t, TRUE);
-	FETCH_FIELD(hv, conf, epilog_slurmctld, charp, FALSE);
-	FETCH_FIELD(hv, conf, ext_sensors_conf, charp, FALSE);
-	FETCH_FIELD(hv, conf, ext_sensors_freq, uint16_t, TRUE);
-	FETCH_FIELD(hv, conf, ext_sensors_type, charp, FALSE);
 
 	FETCH_FIELD(hv, conf, conf_flags, uint32_t, TRUE);
 	FETCH_FIELD(hv, conf, first_job_id, uint32_t, TRUE);
 	FETCH_FIELD(hv, conf, fs_dampening_factor, uint16_t, FALSE);
-	FETCH_FIELD(hv, conf, get_env_timeout, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, gres_plugins, charp, FALSE);
 	FETCH_FIELD(hv, conf, group_time, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, group_force, uint16_t, TRUE);
@@ -477,7 +442,7 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, job_comp_port, uint32_t, TRUE);
 	FETCH_FIELD(hv, conf, job_comp_type, charp, FALSE);
 	FETCH_FIELD(hv, conf, job_comp_user, charp, FALSE);
-	FETCH_FIELD(hv, conf, job_container_plugin, charp, FALSE);
+	FETCH_FIELD(hv, conf, namespace_plugin, charp, FALSE);
 	FETCH_FIELD(hv, conf, job_file_append, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, job_requeue, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, job_submit_plugins, charp, FALSE);
@@ -502,11 +467,9 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, msg_timeout, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, next_job_id, uint32_t, TRUE);
 
-	FETCH_FIELD(hv, conf, node_prefix, charp, FALSE);
 	FETCH_FIELD(hv, conf, over_time_limit, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, plugindir, charp, FALSE);
 	FETCH_FIELD(hv, conf, plugstack, charp, FALSE);
-	FETCH_FIELD(hv, conf, power_parameters, charp, FALSE);
 	FETCH_FIELD(hv, conf, preempt_mode, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, preempt_type, charp, FALSE);
 	FETCH_FIELD(hv, conf, priority_calc_period, uint32_t, TRUE);
@@ -526,9 +489,7 @@ int hv_to_slurm_ctl_conf(HV *hv, slurm_conf_t *conf)
 	FETCH_FIELD(hv, conf, priority_weight_tres, charp, FALSE);
 	FETCH_FIELD(hv, conf, private_data, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, proctrack_type, charp, FALSE);
-	FETCH_FIELD(hv, conf, prolog, charp, FALSE);
 	FETCH_FIELD(hv, conf, prolog_flags, uint16_t, TRUE);
-	FETCH_FIELD(hv, conf, prolog_slurmctld, charp, FALSE);
 
 	FETCH_FIELD(hv, conf, propagate_prio_process, uint16_t, TRUE);
 	FETCH_FIELD(hv, conf, propagate_rlimits, charp, FALSE);
@@ -654,18 +615,3 @@ hv_to_slurmd_status(HV *hv, slurmd_status_t *status)
 
 	return 0;
 }
-
-/*
- * convert perl HV to step_update_request_msg_t
- */
-int
-hv_to_step_update_request_msg(HV *hv, step_update_request_msg_t *update_msg)
-{
-	slurm_init_update_step_msg(update_msg);
-
-	FETCH_FIELD(hv, update_msg, job_id, uint32_t, TRUE);
-	FETCH_FIELD(hv, update_msg, step_id, uint32_t, TRUE);
-	FETCH_FIELD(hv, update_msg, time_limit, uint32_t, TRUE);
-
-	return 0;
-}
