@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  burst_buffer_info.c - get/print the burst buffer state information
  *****************************************************************************
- *  Copyright (C) 2014-2015 SchedMD LLC.
- *  Written by Morris Jette <jette@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -350,6 +349,10 @@ extern void slurm_print_burst_buffer_record(FILE *out,
 	xstrfmtcat(out_buf, "Flags=%s",
 		   slurm_bb_flags2str(burst_buffer_ptr->flags));
 
+	/****** Line ******/
+	xstrcat(out_buf, line_end);
+	xstrfmtcat(out_buf, "PollInterval=%u", burst_buffer_ptr->poll_interval);
+
 	/****** Line ******/
 	xstrcat(out_buf, line_end);
 	xstrfmtcat(out_buf, "StageInTimeout=%u StageOutTimeout=%u ValidateTimeout=%u OtherTimeout=%u",
