@@ -45,7 +45,7 @@
  * IN:  list List of slurmdb_federation_rec_t *
  * RET: SLURM_SUCCESS on success SLURM_ERROR else
  */
-extern int slurmdb_federations_add(void *db_conn, List federation_list)
+extern int slurmdb_federations_add(void *db_conn, list_t *federation_list)
 {
 	if (db_api_uid == -1)
 		db_api_uid = getuid();
@@ -60,9 +60,9 @@ extern int slurmdb_federations_add(void *db_conn, List federation_list)
  * IN:  slurmdb_federation_rec_t  *fed
  * RET: List containing (char *'s) else NULL on error
  */
-extern List slurmdb_federations_modify(void *db_conn,
-				       slurmdb_federation_cond_t *fed_cond,
-				       slurmdb_federation_rec_t *fed)
+extern list_t *slurmdb_federations_modify(void *db_conn,
+					  slurmdb_federation_cond_t *fed_cond,
+					  slurmdb_federation_rec_t *fed)
 {
 	if (db_api_uid == -1)
 		db_api_uid = getuid();
@@ -76,8 +76,8 @@ extern List slurmdb_federations_modify(void *db_conn,
  * IN:  slurmdb_federation_cond_t *fed_cond
  * RET: List containing (char *'s) else NULL on error
  */
-extern List slurmdb_federations_remove(void *db_conn,
-				       slurmdb_federation_cond_t *fed_cond)
+extern list_t *slurmdb_federations_remove(void *db_conn,
+					  slurmdb_federation_cond_t *fed_cond)
 {
 	if (db_api_uid == -1)
 		db_api_uid = getuid();
@@ -91,8 +91,8 @@ extern List slurmdb_federations_remove(void *db_conn,
  * RET: List of slurmdb_federation_rec_t *
  * note List needs to be freed when called
  */
-extern List slurmdb_federations_get(void *db_conn,
-				    slurmdb_federation_cond_t *fed_cond)
+extern list_t *slurmdb_federations_get(void *db_conn,
+				       slurmdb_federation_cond_t *fed_cond)
 {
 	if (db_api_uid == -1)
 		db_api_uid = getuid();
