@@ -481,7 +481,7 @@ extern void print_fields_char_list(print_field_t *field, void *input, int last)
 	xfree(print_this);
 }
 
-static bool _is_wildcard(char *ptr)
+static bool _is_wildcard(char *ptr, job_std_pattern_t *job)
 {
 	switch (*ptr) {
 	case 'A': /* Array job ID */
@@ -489,13 +489,21 @@ static bool _is_wildcard(char *ptr)
 	case 'b': /* Array task ID modulo 10 */
 	case 'J': /* Jobid.stepid */
 	case 'j': /* Job ID */
+	case 'r': /* Restart count */
+	case 'S': /* SLUID */
+	case 's': /* Stepid of the running job */
+	case 'x': /* Job name */
+		return true;
+		break;
 	case 'N': /* Short hostname */
 	case 'n': /* Node id relative to current job */
-	case 's': /* Stepid of the running job */
 	case 't': /* Task id (rank) relative to current job */
 	case 'u': /* User name */
-	case 'x':
-		return true;
+		if (job->is_srun) { /* srun leaves these wildcards untouched */
+			job->io_per_task = true; /* set flag for srun to use */
+			return false;
+		} else
+			return true;
 		break;
 	default:
 		break;
@@ -513,14 +521,17 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 				     job->array_job_id);
 		} else {
 			xstrfmtcatat(*expanded, pos, "%0*u", padding,
-				     job->jobid);
+				     job->step_id.job_id);
 		}
 		break;
 	case 'J': /* Jobid.stepid */
 	case 'j': /* Job ID */
-		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->jobid);
-		if ((*ptr == 'J') && (job->first_step_id != SLURM_BATCH_SCRIPT))
-			xstrfmtcatat(*expanded, pos, ".%d", job->first_step_id);
+		xstrfmtcatat(*expanded, pos, "%0*u", padding,
+			     job->step_id.job_id);
+		if ((*ptr == 'J') &&
+		    (job->step_id.step_id != SLURM_BATCH_SCRIPT))
+			xstrfmtcatat(*expanded, pos, ".%d",
+				     job->step_id.step_id);
 		break;
 	case 'a': /* Array task ID */
 		xstrfmtcatat(*expanded, pos, "%0*u", padding,
@@ -533,21 +544,33 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 	case 'N': /* Short hostname */
 		xstrfmtcatat(*expanded, pos, "%s", job->first_step_node);
 		break;
+	case 'r':
+		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->restart_cnt);
+		break;
+	case 'S': /* SLUID */
+	{
+		char sluid[SLUID_STR_BYTES];
+		print_sluid(job->step_id.sluid, sluid, sizeof(sluid));
+		xstrfmtcatat(*expanded, pos, "%s", sluid);
+		break;
+	}
 	case 's': /* Stepid of the running job */
-		if (job->first_step_id == SLURM_BATCH_SCRIPT)
+		if (job->step_id.step_id == SLURM_BATCH_SCRIPT)
 			xstrcatat(*expanded, pos, "batch");
 		else
 			xstrfmtcatat(*expanded, pos, "%0*u", padding,
-				     job->first_step_id);
+				     job->step_id.step_id);
 		break;
 	case 'n': /* Node id relative to current job */
+		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->nodeid);
+		break;
 	case 't': /* Task id (rank) relative to current job */
-		xstrfmtcatat(*expanded, pos, "0");
+		xstrfmtcatat(*expanded, pos, "%0*u", padding, job->taskid);
 		break;
 	case 'u': /* User name */
 		xstrfmtcatat(*expanded, pos, "%s", job->user);
 		break;
-	case 'x':
+	case 'x': /* Job name */
 		xstrfmtcatat(*expanded, pos, "%s", job->jobname);
 		break;
 	default:
@@ -557,8 +580,6 @@ static void _expand_wildcard(char **expanded, char **pos, char *ptr,
 
 /*
  * Special expansion function for stdin/stdout/stderr filename patterns.
- * Fields that can potentially map to a range of values will use the first in
- * that range (e.g %t is replaced by 0).
  *
  * \      If we found this symbol, don't replace anything.
  * %%     The character "%".
@@ -593,7 +614,7 @@ extern char *expand_stdio_fields(char *stdio_path, job_std_pattern_t *job)
 	if (!stdio_path || !*stdio_path || !job)
 		return NULL;
 
-	if (job->work_dir && (stdio_path[0] != '/')) {
+	if (!job->is_srun && job->work_dir && (stdio_path[0] != '/')) {
 		size_t len = strlen(job->work_dir);
 		if (job->work_dir[len - 1] == '/')
 			xstrcatat(expanded, &pos, job->work_dir);
@@ -630,32 +651,22 @@ extern char *expand_stdio_fields(char *stdio_path, job_std_pattern_t *job)
 			}
 			if (isdigit(*ptr)) {
 				char *tmp_ptr = ptr;
-				if ((padding = strtoul(ptr, &end, 10)) > 9) {
-					/* Remove % and double digit 10 */
-					ptr = end;
+
+				padding = strtoul(ptr, &end, 10);
+				ptr = end;
+				if (padding > 9)
 					padding = 10;
-				} else {
-					ptr++;
-				}
-				/*
-				 * weird behavior fix: we remove all the digits
-				 * except the last one to match with the current
-				 * fname creation.
-				 */
-				if (!_is_wildcard(ptr)) {
-					ptr = tmp_ptr;
-					/* seek until the last digit */
-					while (isdigit(*(ptr + 1))) {
-						ptr++;
-					}
-					xstrfmtcatat(expanded, &pos,
-						     "%c", *ptr);
+
+				if (!_is_wildcard(ptr, job)) {
+					xstrcatat(expanded, &pos, "%");
+					xstrncatat(expanded, &pos, tmp_ptr,
+						   ((ptr + 1) - tmp_ptr));
 					padding = 0;
 					curr_state = STATE_INIT;
 					break;
 				}
 			}
-			if (!_is_wildcard(ptr)) {
+			if (!_is_wildcard(ptr, job)) {
 				/* If not a wildcard print also the %. */
 				xstrfmtcatat(expanded, &pos, "%%%c", *ptr);
 			} else {
