@@ -37,8 +37,10 @@
 
 #include <unistd.h>
 
+#include "slurm/slurm_errno.h"
 #include "slurm/slurm.h"
 
+#include "src/common/http.h"
 #include "src/common/list.h"
 #include "src/common/log.h"
 #include "src/common/xassert.h"
@@ -46,12 +48,14 @@
 #include "src/common/xstring.h"
 #include "src/interfaces/serializer.h"
 
+#include "src/slurmrestd/http.h"
 #include "src/slurmrestd/operations.h"
 #include "src/slurmrestd/rest_auth.h"
 
 static pthread_rwlock_t paths_lock = PTHREAD_RWLOCK_INITIALIZER;
 static list_t *paths = NULL;
 static data_parser_t **parsers; /* symlink to parser array */
+bool inetd_mode = false;
 
 #define MAGIC_HEADER_ACCEPT 0xDF9EAABE
 
@@ -76,11 +80,6 @@ typedef struct {
 	float q; /* quality factor (priority) */
 } http_header_accept_t;
 
-static const char *_name(const on_http_request_args_t *args)
-{
-	return conmgr_fd_get_name(args->context->con);
-}
-
 static void _check_path_magic(const path_t *path)
 {
 	xassert(path->magic == PATH_MAGIC);
@@ -225,37 +224,44 @@ extern int bind_operation_path(const openapi_path_binding_t *op_path,
 	return rc;
 }
 
-static int _operations_router_reject(const on_http_request_args_t *args,
-				     const char *err,
-				     http_status_code_t err_code,
+static int _operations_router_reject(on_http_request_args_t *args,
+				     const char *err, slurm_err_t error_code,
 				     const char *body_encoding)
 {
 	send_http_response_args_t send_args = {
-		.con = args->context->con,
 		.headers = list_create(NULL),
 		.http_major = args->http_major,
 		.http_minor = args->http_minor,
-		.status_code = err_code,
-		.body = err,
-		.body_encoding = (body_encoding ? body_encoding : "text/plain"),
-		.body_length = (err ? strlen(err) : 0),
+		.body_encoding =
+			(body_encoding ? body_encoding : MIME_TYPE_TEXT),
 	};
-	http_header_entry_t close = {
+	http_header_t close = {
+		.magic = HTTP_HEADER_MAGIC,
 		.name = "Connection",
 		.value = "Close",
 	};
 
+	send_args.con = conmgr_fd_get_ref(args->con);
+	send_args.status_code = http_status_from_error(error_code);
+
+	if (!err)
+		send_args.body = slurm_strerror(error_code);
+	else
+		send_args.body = err;
+
+	send_args.body_length = strlen(send_args.body);
+
 	/* Always warn that connection will be closed after the body is sent */
 	list_append(send_args.headers, &close);
 
-	(void) send_http_response(&send_args);
+	(void) send_http_response(args->context, &send_args);
 
 	/* close connection on error */
-	conmgr_queue_close_fd(args->context->con);
+	conmgr_queue_close_fd(send_args.con);
 
 	FREE_NULL_LIST(send_args.headers);
 
-	return SLURM_ERROR;
+	return error_code;
 }
 
 static int _resolve_path(on_http_request_args_t *args, int *path_tag,
@@ -263,9 +269,8 @@ static int _resolve_path(on_http_request_args_t *args, int *path_tag,
 {
 	data_t *path = parse_url_path(args->path, true, false);
 	if (!path)
-		return _operations_router_reject(
-			args, "Unable to parse URL path.",
-			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
+		return _operations_router_reject(args, NULL,
+						 ESLURM_URL_INVALID_PATH, NULL);
 
 	/* attempt to identify path leaf types */
 	(void) data_convert_tree(path, DATA_TYPE_NONE);
@@ -275,15 +280,12 @@ static int _resolve_path(on_http_request_args_t *args, int *path_tag,
 	FREE_NULL_DATA(path);
 
 	if (*path_tag == -1)
-		return _operations_router_reject(
-			args,
-			"Unable find requested URL. Please view /openapi/v3 for API reference.",
-			HTTP_STATUS_CODE_ERROR_NOT_FOUND, NULL);
+		return _operations_router_reject(args, NULL,
+						 ESLURM_REST_UNKNOWN_URL, NULL);
 	else if (*path_tag == -2)
-		return _operations_router_reject(
-			args,
-			"Requested REST method is not defined at URL. Please view /openapi/v3 for API reference.",
-			HTTP_STATUS_CODE_ERROR_METHOD_NOT_ALLOWED, NULL);
+		return _operations_router_reject(args, NULL,
+						 ESLURM_REST_UNKNOWN_URL_METHOD,
+						 NULL);
 	else
 		return SLURM_SUCCESS;
 }
@@ -306,10 +308,11 @@ static int _get_query(on_http_request_args_t *args, data_t **query,
 			query, args->query,
 			(args->query ? strlen(args->query) : 0), read_mime);
 
-	if (rc || !*query)
-		return _operations_router_reject(
-			args, "Unable to parse query.",
-			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
+	if (!rc && !*query)
+		rc = ESLURM_REST_INVALID_QUERY;
+
+	if (rc)
+		return _operations_router_reject(args, NULL, rc, NULL);
 	else
 		return SLURM_SUCCESS;
 
@@ -408,7 +411,7 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 		*read_mime = MIME_TYPE_URL_ENCODED;
 
 		debug4("%s: [%s] did not provide a known content type header. Assuming URL encoded.",
-		       __func__, _name(args));
+		       __func__, args->name);
 	}
 
 	if (args->accept) {
@@ -419,17 +422,17 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 			xassert(ptr->magic == MAGIC_HEADER_ACCEPT);
 
 			debug4("%s: [%s] accepts %s with q=%f",
-			       __func__, _name(args), ptr->type, ptr->q);
+			       __func__, args->name, ptr->type, ptr->q);
 
 			if ((*write_mime = resolve_mime_type(ptr->type,
 							     plugin_ptr))) {
 				debug4("%s: [%s] found accepts %s=%s with q=%f",
-				       __func__, _name(args), ptr->type,
+				       __func__, args->name, ptr->type,
 				       *write_mime, ptr->q);
 				break;
 			} else {
 				debug4("%s: [%s] rejecting accepts %s with q=%f",
-				       __func__, _name(args), ptr->type,
+				       __func__, args->name, ptr->type,
 				       ptr->q);
 			}
 		}
@@ -437,14 +440,13 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 		FREE_NULL_LIST(accept);
 	} else {
 		debug3("%s: [%s] Accept header not specified. Defaulting to JSON.",
-		       __func__, _name(args));
+		       __func__, args->name);
 		*write_mime = MIME_TYPE_JSON;
 	}
 
 	if (!*write_mime)
 		return _operations_router_reject(
-			args, "Accept content type is unknown",
-			HTTP_STATUS_CODE_ERROR_UNSUPPORTED_MEDIA_TYPE, NULL);
+			args, NULL, ESLURM_HTTP_UNKNOWN_ACCEPT_MIME_TYPE, NULL);
 
 	/*
 	 * RFC7230 3.3: Allows for any request to have a BODY but doesn't require
@@ -461,10 +463,9 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 	 * ignored, reject request when both query and body are provided.
 	 */
 	if ((args->body_length > 0) && args->query && args->query[0])
-		return _operations_router_reject(
-			args,
-			"Unexpected HTTP body provided when URL Query provided",
-			HTTP_STATUS_CODE_ERROR_BAD_REQUEST, NULL);
+		return _operations_router_reject(args, NULL,
+						 ESLURM_HTTP_UNEXPECTED_BODY,
+						 NULL);
 
 	if (xstrcasecmp(*read_mime, MIME_TYPE_URL_ENCODED) &&
 	    (args->body_length == 0)) {
@@ -478,14 +479,14 @@ static int _resolve_mime(on_http_request_args_t *args, const char **read_mime,
 		 * requests.
 		 */
 		debug("%s: [%s] Overriding content type from %s to %s for %s",
-		      __func__, _name(args), *read_mime, MIME_TYPE_URL_ENCODED,
+		      __func__, args->name, *read_mime, MIME_TYPE_URL_ENCODED,
 		      get_http_method_string(args->method));
 
 		*read_mime = MIME_TYPE_URL_ENCODED;
 	}
 
 	debug3("%s: [%s] mime read: %s write: %s",
-	       __func__, _name(args), *read_mime, *write_mime);
+	       __func__, args->name, *read_mime, *write_mime);
 
 	return SLURM_SUCCESS;
 }
@@ -499,23 +500,25 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 	int rc;
 	data_t *resp = data_new();
 	char *body = NULL;
-	http_status_code_t e;
+	http_status_code_t e = HTTP_STATUS_CODE_INVALID;
+	void *auth = NULL;
 
 	xassert(op_path);
 	debug3("%s: [%s] BEGIN: calling ctxt handler: 0x%"PRIXPTR"[%d] for path: %s",
-	       __func__, _name(args), (uintptr_t) op_path->callback,
+	       __func__, args->name, (uintptr_t) op_path->callback,
 	       callback_tag, args->path);
 
-	rc = wrap_openapi_ctxt_callback(_name(args), args->method, params,
-					query, callback_tag, resp,
-					args->context->auth, parser, op_path,
-					meta);
+	auth = http_context_set_auth(args->context, NULL);
+
+	rc = wrap_openapi_ctxt_callback(args->name, args->method, params, query,
+					callback_tag, resp, auth, parser,
+					op_path, meta);
 
 	/*
 	 * Clear auth context after callback is complete. Client has to provide
 	 * full auth for every request already.
 	 */
-	FREE_NULL_REST_AUTH(args->context->auth);
+	FREE_NULL_REST_AUTH(auth);
 
 	if (data_get_type(resp) != DATA_TYPE_NULL) {
 		int rc2;
@@ -539,48 +542,17 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 		 *
 		 */
 		send_http_response_args_t send_args = {
-			.con = args->context->con,
 			.http_major = args->http_major,
 			.http_minor = args->http_minor,
 			.status_code = HTTP_STATUS_CODE_REDIRECT_NOT_MODIFIED,
 		};
+		send_args.con = conmgr_fd_get_ref(args->con);
 		e = send_args.status_code;
-		rc = send_http_response(&send_args);
+		rc = send_http_response(args->context, &send_args);
 	} else if (rc && (rc != ESLURM_REST_EMPTY_RESULT)) {
-		e = HTTP_STATUS_CODE_SRVERR_INTERNAL;
-
-		if (rc == ESLURM_REST_INVALID_QUERY)
-			e = HTTP_STATUS_CODE_ERROR_UNPROCESSABLE_CONTENT;
-		else if (rc == ESLURM_REST_FAIL_PARSING)
-			e = HTTP_STATUS_CODE_ERROR_BAD_REQUEST;
-		else if (rc == ESLURM_REST_INVALID_JOBS_DESC)
-			e = HTTP_STATUS_CODE_ERROR_BAD_REQUEST;
-		else if (rc == ESLURM_DATA_UNKNOWN_MIME_TYPE)
-			e = HTTP_STATUS_CODE_ERROR_UNSUPPORTED_MEDIA_TYPE;
-		else if (rc == ESLURM_INVALID_JOB_ID)
-			e = HTTP_STATUS_CODE_ERROR_NOT_FOUND;
-		else if ((rc == SLURM_PROTOCOL_SOCKET_ZERO_BYTES_SENT) ||
-			 (rc == SLURM_COMMUNICATIONS_CONNECTION_ERROR) ||
-			 (rc == SLURM_COMMUNICATIONS_SEND_ERROR) ||
-			 (rc == SLURM_COMMUNICATIONS_RECEIVE_ERROR) ||
-			 (rc == SLURM_COMMUNICATIONS_SHUTDOWN_ERROR) ||
-			 (rc == SLURMCTLD_COMMUNICATIONS_CONNECTION_ERROR) ||
-			 (rc == SLURMCTLD_COMMUNICATIONS_SEND_ERROR) ||
-			 (rc == SLURMCTLD_COMMUNICATIONS_RECEIVE_ERROR) ||
-			 (rc == SLURMCTLD_COMMUNICATIONS_SHUTDOWN_ERROR) ||
-			 (rc == SLURMCTLD_COMMUNICATIONS_BACKOFF) ||
-			 (rc == ESLURM_DB_CONNECTION) ||
-			 (rc == ESLURM_PROTOCOL_INCOMPLETE_PACKET))
-			e = HTTP_STATUS_CODE_SRVERR_BAD_GATEWAY;
-		else if (rc == SLURM_PROTOCOL_SOCKET_IMPL_TIMEOUT)
-			e = HTTP_STATUS_CODE_SRVERR_GATEWAY_TIMEOUT;
-		else if (rc == SLURM_PROTOCOL_AUTHENTICATION_ERROR)
-			e = HTTP_STATUS_CODE_SRVERR_NETWORK_AUTH_REQ;
-
-		rc = _operations_router_reject(args, body, e, write_mime);
+		rc = _operations_router_reject(args, body, rc, write_mime);
 	} else {
 		send_http_response_args_t send_args = {
-			.con = args->context->con,
 			.http_major = args->http_major,
 			.http_minor = args->http_minor,
 			.status_code = HTTP_STATUS_CODE_SUCCESS_OK,
@@ -588,20 +560,23 @@ static int _call_handler(on_http_request_args_t *args, data_t *params,
 			.body_length = 0,
 		};
 
+		send_args.con = conmgr_fd_get_ref(args->con);
+
 		if (body) {
 			send_args.body = body;
 			send_args.body_length = strlen(body);
 			send_args.body_encoding = write_mime;
 		}
 
-		rc = send_http_response(&send_args);
+		rc = send_http_response(args->context, &send_args);
 		e = send_args.status_code;
 	}
 
 	debug3("%s: [%s] END: calling handler: (0x%"PRIXPTR") callback_tag %d for path: %s rc[%d]=%s status[%d]=%s",
-	       __func__, _name(args), (uintptr_t) op_path->callback,
-	       callback_tag, args->path, rc, slurm_strerror(rc), e,
-	       get_http_status_code_string(e));
+	       __func__, args->name, (uintptr_t) op_path->callback,
+	       callback_tag, args->path, rc, slurm_strerror(rc),
+	       ((e == HTTP_STATUS_CODE_INVALID) ? http_status_from_error(rc) :
+		e), get_http_status_code_string(e));
 
 	xfree(body);
 	FREE_NULL_DATA(resp);
@@ -621,15 +596,13 @@ extern int operations_router(on_http_request_args_t *args)
 	data_parser_t *parser = NULL;
 
 	info("%s: [%s] %s %s",
-	     __func__, _name(args), get_http_method_string(args->method),
+	     __func__, args->name, get_http_method_string(args->method),
 	     args->path);
 
 	if ((rc = rest_authenticate_http_request(args))) {
 		error("%s: [%s] authentication failed: %s",
-		      __func__, _name(args), slurm_strerror(rc));
-		_operations_router_reject(args, "Authentication failure",
-					  HTTP_STATUS_CODE_ERROR_UNAUTHORIZED,
-					  NULL);
+		      __func__, args->name, slurm_strerror(rc));
+		_operations_router_reject(args, NULL, rc, NULL);
 		return rc;
 	}
 
@@ -653,7 +626,7 @@ extern int operations_router(on_http_request_args_t *args)
 	slurm_rwlock_unlock(&paths_lock);
 
 	debug5("%s: [%s] found callback handler: (0x%"PRIXPTR") callback_tag=%d path=%s parser=%s",
-	       __func__, _name(args), (uintptr_t) path->op_path->callback,
+	       __func__, args->name, (uintptr_t) path->op_path->callback,
 	       callback_tag, args->path,
 	       (parser ? data_parser_get_plugin(parser) : ""));
 
@@ -671,7 +644,7 @@ cleanup:
 	FREE_NULL_DATA(params);
 
 	/* always clear the auth context */
-	FREE_NULL_REST_AUTH(args->context->auth);
+	http_context_free_null_auth(args->context);
 
 	return rc;
 }
