@@ -52,17 +52,18 @@
 #include <sys/wait.h>
 #include <unistd.h>
 
+#include "src/api/step_launch.h"
+
 #include "src/common/bitstring.h"
-#include "src/interfaces/cli_filter.h"
 #include "src/common/cbuf.h"
 #include "src/common/fd.h"
 #include "src/common/forward.h"
 #include "src/common/hostlist.h"
-#include "src/common/io_hdr.h"
 #include "src/common/log.h"
 #include "src/common/macros.h"
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
+#include "src/common/sluid.h"
 #include "src/common/slurm_opt.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_rlimits_info.h"
@@ -72,14 +73,14 @@
 #include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
-#include "src/api/step_launch.h"
+#include "src/interfaces/cli_filter.h"
 
 #include "src/srun/allocate.h"
 #include "src/srun/debugger.h"
 #include "src/srun/fname.h"
 #include "src/srun/launch.h"
-#include "src/srun/opt.h"
 #include "src/srun/multi_prog.h"
+#include "src/srun/opt.h"
 #include "src/srun/srun_job.h"
 
 /*
@@ -87,7 +88,6 @@
  * about node allocation to be passed to _job_create_structure()
  */
 typedef struct allocation_info {
-	char                   *alias_list;
 	uint16_t               *cpus_per_node;
 	uint32_t               *cpu_count_reps;
 	uint32_t                nnodes;
@@ -107,7 +107,6 @@ typedef struct allocation_info {
 } allocation_info_t;
 
 typedef struct het_job_resp_struct {
-	char **alias_list;
 	uint16_t *cpu_cnt;
 	hostlist_t *host_list;
 	uint32_t node_cnt;
@@ -141,13 +140,12 @@ static void _set_env_vars(resource_allocation_response_msg_t *resp,
 static void _set_env_vars2(resource_allocation_response_msg_t *resp,
 			   int het_job_offset);
 static void _set_ntasks(allocation_info_t *ai, slurm_opt_t *opt_local);
-static void _set_prio_process_env(void);
 static int  _set_rlimit_env(void);
 static void _set_submit_dir_env(void);
 static int  _set_umask_env(void);
 static void _shepherd_notify(int shepherd_fd);
-static int  _shepherd_spawn(srun_job_t *job, List srun_job_list,
-			     bool got_alloc);
+static int _shepherd_spawn(srun_job_t *job, list_t *srun_job_list,
+			   bool got_alloc);
 static void *_srun_signal_mgr(void *no_data);
 static void _srun_cli_filter_post_submit(uint32_t jobid, uint32_t stepid);
 static int  _validate_relative(resource_allocation_response_msg_t *resp,
@@ -168,6 +166,9 @@ job_create_noalloc(void)
 	slurm_opt_t *opt_local = &opt;
 	hostlist_t *hl = hostlist_create(opt_local->nodelist);
 
+	/* Use the reserved slurmdbd cluster_id */
+	sluid_init(0x001, 0);
+
 	if (!hl) {
 		error("Invalid node list `%s' specified", opt_local->nodelist);
 		goto error;
@@ -176,6 +177,7 @@ job_create_noalloc(void)
 	ai->step_id.job_id = MIN_NOALLOC_JOBID +
 		((uint32_t) lrand48() %
 		 (MAX_NOALLOC_JOBID - MIN_NOALLOC_JOBID + 1));
+	ai->step_id.sluid = generate_sluid();
 	ai->step_id.step_id = (uint32_t) (lrand48());
 	ai->step_id.step_het_comp = NO_VAL;
 	ai->nodelist       = opt_local->nodelist;
@@ -187,7 +189,7 @@ job_create_noalloc(void)
 
 	hostlist_destroy(hl);
 
-	cpn[0] = (opt_local->ntasks + ai->nnodes - 1) / ai->nnodes;
+	cpn[0] = ROUNDUP(opt_local->ntasks, ai->nnodes);
 	ai->cpus_per_node  = cpn;
 	cpu_count_reps[0] = ai->nnodes;
 	ai->cpu_count_reps = cpu_count_reps;
@@ -200,6 +202,10 @@ job_create_noalloc(void)
 
 	if (job != NULL)
 		job_update_io_fnames(job, opt_local);
+	if (job && (job->ntasks == NO_VAL)) {
+		job->ntasks = ai->nnodes;
+		job->cpu_count = opt_local->cpus_per_task * job->ntasks;
+	}
 
 error:
 	xfree(ai);
@@ -276,7 +282,6 @@ extern srun_job_t *job_step_create_allocation(
 			slurm_opt_t *opt_local)
 {
 	srun_opt_t *srun_opt = opt_local->srun_opt;
-	uint32_t job_id = resp->job_id;
 	srun_job_t *job = NULL;
 	allocation_info_t *ai = xmalloc(sizeof(allocation_info_t));
 	hostlist_t *hl = NULL;
@@ -286,10 +291,10 @@ extern srun_job_t *job_step_create_allocation(
 	char *step_nodelist = NULL;
 	xassert(srun_opt);
 
-	ai->step_id.job_id          = job_id;
-	ai->step_id.step_id         = NO_VAL;
-	ai->step_id.step_het_comp = NO_VAL;
-	ai->alias_list     = resp->alias_list;
+	ai->step_id = resp->step_id;
+	xassert(ai->step_id.step_id == NO_VAL);
+	xassert(ai->step_id.step_het_comp == NO_VAL);
+
 	if (srun_opt->alloc_nodelist)
 		ai->nodelist = xstrdup(srun_opt->alloc_nodelist);
 	else
@@ -389,12 +394,9 @@ extern srun_job_t *job_step_create_allocation(
 			xfree(opt_local->nodelist);
 			opt_local->nodelist = buf;
 		} else {
-			if (count > ai->nnodes) {
-				/* remove more nodes than needed for
-				 * allocation */
-				int i;
-				for (i = count; i >= ai->nnodes; i--)
-					hostlist_delete_nth(hl, i);
+			/* remove more nodes than needed for allocation */
+			for (int i = count; i > ai->nnodes; i--) {
+				hostlist_delete_nth(hl, i - 1);
 			}
 			xfree(opt_local->nodelist);
 			opt_local->nodelist = hostlist_ranged_string_xmalloc(hl);
@@ -479,7 +481,12 @@ extern srun_job_t *job_step_create_allocation(
 	 * Create job
 	 */
 	job = _job_create_structure(ai, opt_local);
+
+	if (job) {
+		job->use_protocol_ver = resp->start_protocol_ver;
+	}
 error:
+	xfree(ai->nodelist);
    	xfree(ai);
 	return (job);
 
@@ -495,13 +502,13 @@ extern srun_job_t *job_create_allocation(
 	srun_job_t *job;
 	allocation_info_t *i = xmalloc(sizeof(allocation_info_t));
 
-	i->alias_list     = resp->alias_list;
+	i->step_id = resp->step_id;
+	xassert(i->step_id.step_id == NO_VAL);
+	xassert(i->step_id.step_het_comp == NO_VAL);
+
 	i->nodelist       = _normalize_hostlist(resp->node_list);
 	i->nnodes	  = resp->node_cnt;
 	i->partition      = resp->partition;
-	i->step_id.job_id          = resp->job_id;
-	i->step_id.step_id         = NO_VAL;
-	i->step_id.step_het_comp = NO_VAL;
 	i->num_cpu_groups = resp->num_cpu_groups;
 	i->cpus_per_node  = resp->cpus_per_node;
 	i->cpu_count_reps = resp->cpu_count_reps;
@@ -519,6 +526,7 @@ extern srun_job_t *job_create_allocation(
 		job->account = xstrdup(resp->account);
 		job->qos = xstrdup(resp->qos);
 		job->resv_name = xstrdup(resp->resv_name);
+		job->use_protocol_ver = resp->start_protocol_ver;
 	}
 
 	xfree(i->nodelist);
@@ -527,9 +535,9 @@ extern srun_job_t *job_create_allocation(
 	return (job);
 }
 
-static void _copy_args(List missing_argc_list, slurm_opt_t *opt_master)
+static void _copy_args(list_t *missing_argc_list, slurm_opt_t *opt_master)
 {
-	ListIterator iter;
+	list_itr_t *iter;
 	slurm_opt_t *opt_local;
 	int i;
 
@@ -550,12 +558,12 @@ static void _copy_args(List missing_argc_list, slurm_opt_t *opt_master)
  * rebuilt for multiple option structures ("--het-group=1,2" becomes two
  * opt structures). Clear "het_grp_bits".if determined to not be a hetjob.
  */
-static void _het_grp_test(List opt_list)
+static void _het_grp_test(list_t *opt_list)
 {
-	ListIterator iter;
+	list_itr_t *iter;
 	int het_job_offset;
 	bitstr_t *master_map = NULL;
-	List missing_argv_list = NULL;
+	list_t *missing_argv_list = NULL;
 	bool multi_comp = false, multi_prog = false;
 
 	if (opt_list) {
@@ -617,10 +625,10 @@ static void _het_grp_test(List opt_list)
  * Copy job name from last component to all hetjob components unless
  * explicitly set.
  */
-static void _match_job_name(List opt_list)
+static void _match_job_name(list_t *opt_list)
 {
 	int cnt;
-	ListIterator iter;
+	list_itr_t *iter;
 	slurm_opt_t *opt_local;
 
 	if (!opt_list)
@@ -658,7 +666,7 @@ static int _sort_by_offset(void *x, void *y)
 	return 0;
 }
 
-static void _post_opts(List opt_list)
+static void _post_opts(list_t *opt_list)
 {
 	_het_grp_test(opt_list);
 	_match_job_name(opt_list);
@@ -687,7 +695,7 @@ extern void init_srun(int argc, char **argv, log_options_t *logopt,
 	 * Initialize plugin stack, read options from plugins, etc.
 	 */
 	init_spank_env();
-	if (spank_init(NULL) < 0) {
+	if (spank_init(NULL)) {
 		error("Plug-in initialization failed");
 		exit(error_exit);
 	}
@@ -726,6 +734,7 @@ extern void init_srun(int argc, char **argv, log_options_t *logopt,
 			 */
 			het_job_argc -= het_job_argc_off;
 			het_job_argv += het_job_argc_off;
+			colon_cnt++;
 		} else {
 			het_job_fini = true;
 		}
@@ -755,7 +764,7 @@ extern void init_srun(int argc, char **argv, log_options_t *logopt,
 	}
 
 	(void) _set_rlimit_env();
-	_set_prio_process_env();
+	set_prio_process_env();
 	(void) _set_umask_env();
 	_set_submit_dir_env();
 
@@ -766,11 +775,14 @@ extern void init_srun(int argc, char **argv, log_options_t *logopt,
 }
 
 /*
- * Modify options for a job step (after job allocaiton is complete
+ * Modify options for a job step (after job allocation is complete
  */
-static void _set_step_opts(slurm_opt_t *opt_local)
+static void _set_step_opts(slurm_opt_t *opt_local,
+			   resource_allocation_response_msg_t *resp)
 {
 	srun_opt_t *srun_opt = opt_local->srun_opt;
+	int new_cpt;
+
 	xassert(srun_opt);
 
 	opt_local->time_limit = NO_VAL;/* not applicable for step, only job */
@@ -784,6 +796,15 @@ static void _set_step_opts(slurm_opt_t *opt_local)
 		/* Step gets all CPUs in the job allocation. */
 		srun_opt->exclusive = false;
 	}
+
+	new_cpt = slurm_opt_get_tres_per_task_cpu_cnt(resp->tres_per_task);
+	if (new_cpt)
+		opt_local->cpus_per_task = new_cpt;
+
+	if (resp->tres_per_task) {
+		xfree(opt_local->tres_per_task);
+		SWAP(opt_local->tres_per_task, resp->tres_per_task);
+	}
 }
 
 static int _handle_het_step_exclude(srun_job_t *job, slurm_opt_t *opt_local,
@@ -844,15 +865,16 @@ end_it:
  * the job allocation request with its requested options.
  */
 static int _create_job_step(srun_job_t *job, bool use_all_cpus,
-			    List srun_job_list, uint32_t het_job_id,
+			    list_t *srun_job_list, uint32_t het_job_id,
 			    char *het_job_nodelist)
 {
-	ListIterator opt_iter = NULL, job_iter;
+	list_itr_t *opt_iter = NULL, *job_iter;
 	slurm_opt_t *opt_local = &opt;
 	uint32_t node_offset = 0, het_job_nnodes = 0, step_id = NO_VAL;
 	uint32_t het_job_ntasks = 0, task_offset = 0;
 	bool update_het_nnodes = false;
 	uint32_t updated_het_nnodes;
+	uint32_t updated_het_ntasks = 0;
 
 	job_step_create_response_msg_t *step_resp;
 	char *resv_ports = NULL;
@@ -883,7 +905,10 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 				job->step_id.step_het_comp = NO_VAL;
 
 			het_job_nnodes += job->nhosts;
-			het_job_ntasks += job->ntasks;
+			if (job->ntasks == NO_VAL)
+				het_job_ntasks = NO_VAL;
+			else if (het_job_ntasks != NO_VAL)
+				het_job_ntasks += job->ntasks;
 		}
 
 		updated_het_nnodes = het_job_nnodes;
@@ -929,7 +954,7 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 			task_offset += job->ntasks;
 
 			/*
-			 * If packing nodes (CR_PACK_NODES, -mpack), the step
+			 * If packing nodes (SELECT_PACK_NODES, -mpack), the step
 			 * may have an updated layout. Need to update each
 			 * component's het_job_nnodes with the updated counts.
 			 */
@@ -937,6 +962,9 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 				update_het_nnodes = true;
 				updated_het_nnodes -= old_nhosts - job->nhosts;
 			}
+
+			if (het_job_ntasks == NO_VAL)
+				updated_het_ntasks += job->ntasks;
 		}
 
 		if (update_het_nnodes) {
@@ -945,6 +973,12 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 				job->het_job_nnodes = updated_het_nnodes;
 			}
 		}
+		if (updated_het_ntasks) {
+			list_iterator_reset(job_iter);
+			while ((job = list_next(job_iter))) {
+				job->het_job_ntasks = updated_het_ntasks;
+			}
+		}
 
 		FREE_NULL_HOSTLIST(exclude_hl);
 
@@ -987,16 +1021,29 @@ static int _create_job_step(srun_job_t *job, bool use_all_cpus,
 			job->het_job_ntasks = job->ntasks;
 			job->het_job_task_offset = 0;
 		}
-		return create_job_step(job, use_all_cpus, &opt);
+		if ((rc = create_job_step(job, use_all_cpus, &opt)) < 0)
+			return rc;
+
+		if (het_job_id) {
+			/*
+			 * If packing nodes (SELECT_PACK_NODES, -mpack), the step
+			 * may have an updated layout.
+			 */
+			job->het_job_nnodes = job->nhosts;
+			/* The stepmgr logic can modify ntasks */
+			job->het_job_ntasks = job->ntasks;
+		}
+
+		return rc;
 	} else {
 		return -1;
 	}
 }
 
-static void _cancel_steps(List srun_job_list)
+static void _cancel_steps(list_t *srun_job_list)
 {
 	srun_job_t *job;
-	ListIterator job_iter;
+	list_itr_t *job_iter;
 	slurm_msg_t req;
 	step_complete_msg_t msg;
 	int rc = 0;
@@ -1026,32 +1073,25 @@ static void _cancel_steps(List srun_job_list)
 static void _het_job_struct_del(void *x)
 {
 	het_job_resp_struct_t *het_job_resp = (het_job_resp_struct_t *) x;
-	int i;
 
-	if (het_job_resp->alias_list) {
-		for (i = 0; i < het_job_resp->node_cnt; i++)
-			xfree(het_job_resp->alias_list[i]);
-		xfree(het_job_resp->alias_list);
-	}
 	xfree(het_job_resp->cpu_cnt);
 	if (het_job_resp->host_list)
 		hostlist_destroy(het_job_resp->host_list);
 	xfree(het_job_resp);
 }
 
-static char *_compress_het_job_nodelist(List used_resp_list)
+static char *_compress_het_job_nodelist(list_t *used_resp_list)
 {
 	resource_allocation_response_msg_t *resp;
 	het_job_resp_struct_t *het_job_resp;
-	List het_job_resp_list;
-	ListIterator resp_iter;
-	char *aliases = NULL, *save_ptr = NULL, *tok, *tmp;
+	list_t *het_job_resp_list;
+	list_itr_t *resp_iter;
+	char *tmp;
 	char *het_job_nodelist = NULL, *node_name;
 	hostset_t *hs;
 	int cnt, i, j, k;
 	uint16_t *cpus;
 	uint32_t *reps, cpu_inx;
-	bool have_aliases = false;
 
 	if (!used_resp_list)
 		return het_job_nodelist;
@@ -1066,27 +1106,6 @@ static char *_compress_het_job_nodelist(List used_resp_list)
 		hostset_insert(hs, resp->node_list);
 		het_job_resp = xmalloc(sizeof(het_job_resp_struct_t));
 		het_job_resp->node_cnt = resp->node_cnt;
-		/*
-		 * alias_list contains <NodeName>:<NodeAddr>:<NodeHostName>
-		 * values in comma separated list
-		 */
-		if (resp->alias_list) {
-			have_aliases = true;
-			het_job_resp->alias_list = xmalloc(sizeof(char *) *
-							   resp->node_cnt);
-			tmp = xstrdup(resp->alias_list);
-			i = 0;
-			tok = strtok_r(tmp, ",", &save_ptr);
-			while (tok) {
-				if (i >= resp->node_cnt) {
-					fatal("%s: Invalid alias_list",
-					      __func__);
-				}
-				het_job_resp->alias_list[i++] = xstrdup(tok);
-				tok = strtok_r(NULL, ",", &save_ptr);
-			}
-			xfree(tmp);
-		}
 		het_job_resp->cpu_cnt =
 			xmalloc(sizeof(uint16_t) * resp->node_cnt);
 		het_job_resp->host_list = hostlist_create(resp->node_list);
@@ -1118,19 +1137,6 @@ static char *_compress_het_job_nodelist(List used_resp_list)
 			j = hostlist_find(het_job_resp->host_list, node_name);
 			if ((j == -1) || !het_job_resp->cpu_cnt)
 				continue;	/* node not in this hetjob */
-			if (have_aliases) {
-				if (aliases)
-					xstrcat(aliases, ",");
-				if (het_job_resp->alias_list &&
-				    het_job_resp->alias_list[j]) {
-					xstrcat(aliases,
-						het_job_resp->alias_list[j]);
-				} else {
-					xstrfmtcat(aliases, "%s:%s:%s",
-						   node_name, node_name,
-						   node_name);
-				}
-			}
 			if (cpus[cpu_inx] == het_job_resp->cpu_cnt[j]) {
 				reps[cpu_inx]++;
 			} else {
@@ -1166,7 +1172,7 @@ static char *_compress_het_job_nodelist(List used_resp_list)
  * that allocation. So here we will copy the resp_list to the number of
  * components we care about.
  */
-static void _copy_job_resp(List job_resp_list, int count)
+static void _copy_job_resp(list_t *job_resp_list, int count)
 {
 	resource_allocation_response_msg_t *new, *orig;
 	xassert(job_resp_list);
@@ -1199,19 +1205,17 @@ static void _check_gpus_per_socket(slurm_opt_t *opt_local)
 extern void create_srun_job(void **p_job, bool *got_alloc)
 {
 	resource_allocation_response_msg_t *resp;
-	List job_resp_list = NULL, srun_job_list = NULL;
-	List used_resp_list = NULL;
-	ListIterator opt_iter, resp_iter;
+	list_t *job_resp_list = NULL, *srun_job_list = NULL;
+	list_t *used_resp_list = NULL;
+	list_itr_t *opt_iter, *resp_iter;
 	srun_job_t *job = NULL;
 	int i, max_list_offset, max_het_job_offset, het_job_offset = -1,
 		het_step_offset = -1;
-	uint32_t my_job_id = 0, het_job_id = 0;
+	slurm_step_id_t my_step_id = SLURM_STEP_ID_INITIALIZER;
+	uint32_t het_job_id = 0;
 	char *het_job_nodelist = NULL;
 	bool begin_error_logged = false;
 	bool core_spec_error_logged = false;
-#ifdef HAVE_NATIVE_CRAY
-	bool network_error_logged = false;
-#endif
 	bool node_cnt_error_logged = false;
 	bool tres_license_error_logged = false;
 	bool x11_error_logged = false;
@@ -1286,8 +1290,8 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 		resp_iter = list_iterator_create(job_resp_list);
 		while ((resp = list_next(resp_iter))) {
 			bool merge_nodelist = true;
-			if (my_job_id == 0) {
-				my_job_id = resp->job_id;
+			if (my_step_id.job_id == NO_VAL) {
+				my_step_id = resp->step_id;
 				if (resp->working_cluster_rec)
 					slurm_setup_remote_working_cluster(resp);
 			}
@@ -1359,31 +1363,15 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 					      "at job allocation time.");
 					core_spec_error_logged = true;
 				}
-#ifdef HAVE_NATIVE_CRAY
-				if (opt_local->network &&
-				    !network_error_logged) {
-					if (slurm_option_set_by_env(opt_local,
-								    LONG_OPT_NETWORK)) {
-						debug2("Ignoring SLURM_NETWORK value for a "
-						       "job step within an existing job. "
-						       "Using what was set at job "
-						       "allocation time.  Most likely this "
-						       "variable was set by sbatch or salloc.");
-					} else {
-						error("Ignoring --network value for a job step "
-						      "within an existing job. Set network "
-						      "options at job allocation time.");
-					}
-					network_error_logged = true;
-				}
-				xfree(opt_local->network);
+
 				/*
 				 * Here we send the het job groups to the
 				 * slurmctld to set up the interconnect
 				 * correctly.  We only ever need to send it to
 				 * the first component of the step.
+				 *
+				 * FIXME - is this still needed post-Cray?
 				 */
-#endif
 				if (g_het_grp_bits) {
 					xfree(opt_local->step_het_grps);
 					opt_local->step_het_grps =
@@ -1431,8 +1419,8 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 			FREE_NULL_LIST(srun_job_list);	/* Just use "job" */
 		if (list_count(job_resp_list) > 1) {
 			/* only set if actually a hetjob */
-			if (!local_het_step && my_job_id)
-				het_job_id = my_job_id;
+			if (!local_het_step && (my_step_id.job_id != NO_VAL))
+				het_job_id = my_step_id.job_id;
 			het_job_nodelist =
 				_compress_het_job_nodelist(used_resp_list);
 		}
@@ -1440,7 +1428,7 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 		if (_create_job_step(job, false, srun_job_list, het_job_id,
 				     het_job_nodelist) < 0) {
 			if (*got_alloc)
-				slurm_complete_job(my_job_id, 1);
+				slurm_complete_job(&my_step_id, 1);
 			else
 				_cancel_steps(srun_job_list);
 			exit(error_exit);
@@ -1448,19 +1436,16 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 		xfree(het_job_nodelist);
 	} else {
 		/* Combined job allocation and job step launch */
-#if defined HAVE_FRONT_END
-		uid_t my_uid = getuid();
-		if ((my_uid != 0) && (my_uid != slurm_conf.slurm_user_id)) {
-			error("srun task launch not supported on this system");
-			exit(error_exit);
-		}
-#endif
 		if (slurm_option_set_by_cli(&opt, 'J'))
 			setenvfs("SLURM_JOB_NAME=%s", opt.job_name);
 		else if (!slurm_option_set_by_env(&opt, 'J') && opt.argc)
 			setenvfs("SLURM_JOB_NAME=%s", opt.argv[0]);
 
 		if (opt_list) {
+			if (!colon_cnt) {
+				error("--het-group expected to be used within an HetJob allocation");
+				exit(error_exit);
+			}
 			job_resp_list = allocate_het_job_nodes();
 			if (!job_resp_list)
 				exit(error_exit);
@@ -1470,8 +1455,8 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 			while ((resp = list_next(resp_iter))) {
 				slurm_opt_t *opt_local;
 
-				if (my_job_id == 0) {
-					my_job_id = resp->job_id;
+				if (my_step_id.job_id == NO_VAL) {
+					my_step_id = resp->step_id;
 					*got_alloc = true;
 				}
 				opt_local = list_next(opt_iter);
@@ -1481,13 +1466,13 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 				_set_env_vars(resp, ++het_job_offset);
 				_set_env_vars2(resp, het_job_offset);
 				if (_validate_relative(resp, opt_local)) {
-					slurm_complete_job(my_job_id, 1);
+					slurm_complete_job(&my_step_id, 1);
 					exit(error_exit);
 				}
 				job = job_create_allocation(resp, opt_local);
 				job->het_job_offset = het_job_offset;
 				list_append(srun_job_list, job);
-				_set_step_opts(opt_local);
+				_set_step_opts(opt_local, resp);
 			}
 			list_iterator_destroy(opt_iter);
 			list_iterator_destroy(resp_iter);
@@ -1499,31 +1484,38 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 					 het_job_offset + 1);
 			}
 		} else {
+			if (sropt.het_grp_bits &&
+			    (bit_fls(sropt.het_grp_bits) != -1)) {
+				error("--het-group expected to be used within an HetJob allocation");
+				exit(error_exit);
+			}
+
 			if (!(resp = allocate_nodes(&opt)))
 				exit(error_exit);
 			*got_alloc = true;
-			my_job_id = resp->job_id;
+			my_step_id = resp->step_id;
 			_print_job_information(resp);
 			_set_env_vars(resp, -1);
 			if (_validate_relative(resp, &opt)) {
-				slurm_complete_job(resp->job_id, 1);
+				slurm_complete_job(&resp->step_id, 1);
 				exit(error_exit);
 			}
 			job = job_create_allocation(resp, &opt);
-			_set_step_opts(&opt);
+			_set_step_opts(&opt, resp);
 		}
 		if (srun_job_list && (list_count(srun_job_list) > 1) &&
-		    opt_list && (list_count(opt_list) > 1) && my_job_id) {
+		    opt_list && (list_count(opt_list) > 1) &&
+		    (my_step_id.job_id != NO_VAL)) {
 			/* only set if actually a hetjob */
 			if (!local_het_step)
-				het_job_id = my_job_id;
+				het_job_id = my_step_id.job_id;
 			het_job_nodelist =
 				_compress_het_job_nodelist(job_resp_list);
 		}
 
 		if (_create_job_step(job, true, srun_job_list, het_job_id,
 				     het_job_nodelist) < 0) {
-			slurm_complete_job(my_job_id, 1);
+			slurm_complete_job(&my_step_id, 1);
 			exit(error_exit);
 		}
 		xfree(het_job_nodelist);
@@ -1552,7 +1544,8 @@ extern void create_srun_job(void **p_job, bool *got_alloc)
 		*p_job = (void *) job;
 
 	if (job)
-	        _srun_cli_filter_post_submit(my_job_id, job->step_id.step_id);
+		_srun_cli_filter_post_submit(my_step_id.job_id,
+					     job->step_id.step_id);
 }
 
 extern void pre_launch_srun_job(srun_job_t *job, slurm_opt_t *opt_local)
@@ -1561,7 +1554,7 @@ extern void pre_launch_srun_job(srun_job_t *job, slurm_opt_t *opt_local)
 		slurm_thread_create(&signal_thread, _srun_signal_mgr, job);
 
 	_run_srun_prolog(job);
-	if (_call_spank_local_user(job, opt_local) < 0) {
+	if (_call_spank_local_user(job, opt_local)) {
 		error("Failure in local plugin stack");
 		slurm_step_launch_abort(job->step_ctx);
 		exit(error_exit);
@@ -1577,16 +1570,16 @@ extern void fini_srun(srun_job_t *job, bool got_alloc, uint32_t *global_rc)
 
 		/* Tell slurmctld that we were cancelled */
 		if (job->state >= SRUN_JOB_CANCELLED)
-			slurm_complete_job(job->step_id.job_id, NO_VAL);
+			slurm_complete_job(&job->step_id, NO_VAL);
 		else
-			slurm_complete_job(job->step_id.job_id, *global_rc);
+			slurm_complete_job(&job->step_id, *global_rc);
 	}
 	_shepherd_notify(shepherd_fd);
 
 	if (signal_thread) {
 		srun_shutdown = true;
 		pthread_kill(signal_thread, SIGINT);
-		pthread_join(signal_thread,  NULL);
+		slurm_thread_join(signal_thread);
 	}
 
 	_run_srun_epilog(job);
@@ -1644,8 +1637,7 @@ job_force_termination(srun_job_t *job)
 		}
 		if (kill_sent == 1) {
 			/* Try sending SIGKILL through slurmctld */
-			slurm_kill_job_step(job->step_id.job_id,
-					    job->step_id.step_id, SIGKILL, 0);
+			slurm_kill_job_step(&job->step_id, SIGKILL, 0);
 		}
 	}
 	kill_sent++;
@@ -1655,20 +1647,19 @@ static void _set_ntasks(allocation_info_t *ai, slurm_opt_t *opt_local)
 {
 	int cnt = 0;
 
-	if (opt_local->ntasks_set && (opt_local->ntasks_per_node == NO_VAL))
+	/* Distinction between explicit or implicit set of ntasks */
+	if (opt_local->ntasks_opt_set ||
+	    (opt_local->ntasks_set &&
+	     (opt_local->ntasks_per_node == NO_VAL)))
 		return;
 
 	if (opt_local->ntasks_per_node != NO_VAL) {
 		cnt = ai->nnodes * opt_local->ntasks_per_node;
 		opt_local->ntasks_set = true;	/* implicit */
 	} else if (opt_local->cpus_set) {
-		int i;
-
-		for (i = 0; i < ai->num_cpu_groups; i++)
-			cnt += (ai->cpu_count_reps[i] *
-				(ai->cpus_per_node[i] /
-				 opt_local->cpus_per_task));
+		opt_local->ntasks = NO_VAL;
 		opt_local->ntasks_set = true;	/* implicit */
+		return;
 	}
 
 	opt_local->ntasks = (cnt < ai->nnodes) ? ai->nnodes : cnt;
@@ -1690,7 +1681,6 @@ static srun_job_t *_job_create_structure(allocation_info_t *ainfo,
 	slurm_cond_init(&job->state_cond, NULL);
 	job->state = SRUN_JOB_INIT;
 
- 	job->alias_list = xstrdup(ainfo->alias_list);
 	job->container = xstrdup(opt_local->container);
  	job->nodelist = xstrdup(ainfo->nodelist);
  	job->partition = xstrdup(ainfo->partition);
@@ -1702,10 +1692,6 @@ static srun_job_t *_job_create_structure(allocation_info_t *ainfo,
 	job->het_job_task_offset = NO_VAL;
 	job->nhosts   = ainfo->nnodes;
 
-#if defined HAVE_FRONT_END
-	/* Limited job step support */
-	opt_local->overcommit = true;
-#else
 	if (opt_local->min_nodes > job->nhosts) {
 		error("Only allocated %d nodes asked for %d",
 		      job->nhosts, opt_local->min_nodes);
@@ -1723,7 +1709,7 @@ static srun_job_t *_job_create_structure(allocation_info_t *ainfo,
 		xfree(job);
 		return NULL;
 	}
-#endif
+
 	job->ntasks  = opt_local->ntasks;
 	job->ntasks_per_board = ainfo->ntasks_per_board;
 	job->ntasks_per_core = ainfo->ntasks_per_core;
@@ -1734,9 +1720,13 @@ static srun_job_t *_job_create_structure(allocation_info_t *ainfo,
 	 * requested step (we might very well use less, especially if
 	 * --exclusive is used).  Else get the total for the allocation given.
 	 */
-	if (opt_local->cpus_set)
-		job->cpu_count = opt_local->ntasks * opt_local->cpus_per_task;
-	else {
+	if (opt_local->cpus_set) {
+		if (opt_local->ntasks == NO_VAL)
+			job->cpu_count = NO_VAL;
+		else
+			job->cpu_count = opt_local->ntasks *
+				opt_local->cpus_per_task;
+	} else {
 		for (i = 0; i < ainfo->num_cpu_groups; i++) {
 			job->cpu_count += ainfo->cpus_per_node[i] *
 				ainfo->cpu_count_reps[i];
@@ -1850,7 +1840,7 @@ static void _print_job_information(resource_allocation_response_msg_t *resp)
 		return;
 
 	xstrfmtcat(str, "jobid %u: nodes(%u):`%s', cpu counts: ",
-		   resp->job_id, resp->node_cnt, resp->node_list);
+		   resp->step_id.job_id, resp->node_cnt, resp->node_list);
 
 	for (i = 0; i < resp->num_cpu_groups; i++) {
 		xstrfmtcat(str, "%s%u(x%u)",
@@ -2012,6 +2002,15 @@ static void _set_env_vars(resource_allocation_response_msg_t *resp,
 		xfree(key);
 	}
 
+	if (resp->segment_size) {
+		key = _build_key("SLURM_JOB_SEGMENT_SIZE", het_job_offset);
+		if (!getenv(key) &&
+		    (setenvf(NULL, key, "%u", resp->segment_size) < 0)) {
+			error("unable to set %s in environment", key);
+		}
+		xfree(key);
+	}
+
 	return;
 }
 
@@ -2034,7 +2033,7 @@ static void _set_env_vars2(resource_allocation_response_msg_t *resp,
 
 	key = _build_key("SLURM_JOB_ID", het_job_offset);
 	if (!getenv(key) &&
-	    (setenvf(NULL, key, "%u", resp->job_id) < 0)) {
+	    (setenvf(NULL, key, "%u", resp->step_id.job_id) < 0)) {
 		error("unable to set %s in environment", key);
 	}
 	xfree(key);
@@ -2072,34 +2071,6 @@ static void _set_env_vars2(resource_allocation_response_msg_t *resp,
 	}
 }
 
-/*
- * _set_prio_process_env
- *
- * Set the internal SLURM_PRIO_PROCESS environment variable to support
- * the propagation of the users nice value and the "PropagatePrioProcess"
- * config keyword.
- */
-static void  _set_prio_process_env(void)
-{
-	int retval;
-
-	errno = 0; /* needed to detect a real failure since prio can be -1 */
-
-	if ((retval = getpriority (PRIO_PROCESS, 0)) == -1)  {
-		if (errno) {
-			error ("getpriority(PRIO_PROCESS): %m");
-			return;
-		}
-	}
-
-	if (setenvf (NULL, "SLURM_PRIO_PROCESS", "%d", retval) < 0) {
-		error ("unable to set SLURM_PRIO_PROCESS in environment");
-		return;
-	}
-
-	debug ("propagating SLURM_PRIO_PROCESS=%d", retval);
-}
-
 /* Set SLURM_RLIMIT_* environment variables with current resource
  * limit values, reset RLIMIT_NOFILE to maximum possible value */
 static int _set_rlimit_env(void)
@@ -2226,7 +2197,8 @@ static void _shepherd_notify(int shepherd_fd)
 	close(shepherd_fd);
 }
 
-static int _shepherd_spawn(srun_job_t *job, List srun_job_list, bool got_alloc)
+static int _shepherd_spawn(srun_job_t *job, list_t *srun_job_list,
+			   bool got_alloc)
 {
 	int shepherd_pipe[2], rc;
 	pid_t shepherd_pid;
@@ -2263,20 +2235,18 @@ static int _shepherd_spawn(srun_job_t *job, List srun_job_list, bool got_alloc)
 	}
 
 	if (srun_job_list) {
-		ListIterator job_iter;
+		list_itr_t *job_iter;
 		job_iter  = list_iterator_create(srun_job_list);
 		while ((job = list_next(job_iter))) {
-			(void) slurm_kill_job_step(job->step_id.job_id, job->step_id.step_id,
-						   SIGKILL, 0);
+			(void) slurm_kill_job_step(&job->step_id, SIGKILL, 0);
 			if (got_alloc)
-				slurm_complete_job(job->step_id.job_id, NO_VAL);
+				slurm_complete_job(&job->step_id, NO_VAL);
 		}
 		list_iterator_destroy(job_iter);
 	} else {
-		(void) slurm_kill_job_step(job->step_id.job_id,
-					   job->step_id.step_id, SIGKILL, 0);
+		(void) slurm_kill_job_step(&job->step_id, SIGKILL, 0);
 		if (got_alloc)
-			slurm_complete_job(job->step_id.job_id, NO_VAL);
+			slurm_complete_job(&job->step_id, NO_VAL);
 	}
 
 	_exit(0);
